# coding: utf-8
# !/usr/bin/python
"""
@File       :   cylinder_qrcode.py
@Author     :   jiaming
@Modify Time:   2020/2/21 15:52
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :
            准备二维码模板。
            拉伸图片。
"""
import math
from copy import deepcopy
from PIL import Image

from app.static_data import dataPath, standard_module_table

# 二维码版本
VERSION = -1
# 二维码尺寸
SIZE = -1
# 每个像素由 4*4 个像素点构成
BOX = 4
# 边界距离 单位
BOUNDARY = 4

# 标线坐标
standard_line = []
# 写入数据的坐标
data_list_right = []
data_list_left = []


def draw_position_detection_pattern(figure=None, key=-1):
    """
    绘制position detection pattern
    :param img:
    :return:
    """
    coordinates = []
    # 绘制第一个图案
    for i in range(7):
        coordinates.append((BOUNDARY+i, BOUNDARY))
    for i in range(6):
        coordinates.append((BOUNDARY, BOUNDARY+i))
        coordinates.append((BOUNDARY+6, BOUNDARY+i))
    for i in range(7):
        coordinates.append((BOUNDARY+i, BOUNDARY+6))
    for i in range(3):
        for j in range(3):
            coordinates.append((BOUNDARY+2+i, BOUNDARY+2+j))
    # 绘制第二个图案
    column = BOUNDARY + SIZE - 7
    for i in range(7):
        coordinates.append((column + i, BOUNDARY))
    for i in range(6):
        coordinates.append((column, BOUNDARY + i))
        coordinates.append((column + 6, BOUNDARY + i))
    for i in range(7):
        coordinates.append((column + i, BOUNDARY + 6))
    for i in range(3):
        for j in range(3):
            coordinates.append((column + 2 + i, BOUNDARY + 2 + j))

    for value in coordinates:
        i, j = value
        for k in range(0, BOX, 1):
            for z in range(0, BOX, 1):
                figure.putpixel((i * BOX + k, j * BOX + z), (0, 0, 0))
    if key != -1:
        for i in range(BOUNDARY, BOUNDARY+8):
            for j in range(BOUNDARY, BOUNDARY+8):
                for k in range(0, BOX, 1):
                    for z in range(0, BOX, 1):
                        figure.putpixel((i * BOX + k, j * BOX + z), (0, 0,0))
        for i in range(BOUNDARY + SIZE - 8, BOUNDARY + SIZE):
            for j in range(BOUNDARY, BOUNDARY+8):
                for k in range(0, BOX, 1):
                    for z in range(0, BOX, 1):
                        figure.putpixel((i * BOX + k, j * BOX + z), (0, 0,0))
    return figure


def draw_alignment_pattern(figure=None, key=-1):
    """
    绘制 alignment pattern
    :param img:
    :return:
    """
    if VERSION <= 3:
        pass
    elif 3 < VERSION <= 6:
        x = (SIZE+BOUNDARY*2) // 2
        y = (SIZE+BOUNDARY*2) // 2
        for k in range(0, BOX, 1):
            for z in range(0, BOX, 1):
                figure.putpixel((x * BOX + k, y * BOX + z), (0, 0, 0))
        temp_list = []
        for i in range(-2, 2):
            temp_list.append((x + i, y - 2))
        for i in range(-2, 3):
            temp_list.append((x - 2, y + i))
            temp_list.append((x + 2, y + i))
        for i in range(-2, 2):
            temp_list.append((x + i, y + 2))
        for value in temp_list:
            i, j = value
            for k in range(0, BOX, 1):
                for z in range(0, BOX, 1):
                    figure.putpixel((i * BOX + k, j * BOX + z), (0, 0, 0))
        if key != -1:
            for i in range(-2, 3):
                for j in range(-2, 3):
                    for k in range(0, BOX, 1):
                        for z in range(0, BOX, 1):
                            figure.putpixel(((x+i) * BOX + k, (y+j) * BOX + z),
                                            (0, 0, 0))
    elif 6 < VERSION <= 10:
        _, tuple = standard_module_table[VERSION]
        x1, x2, x3 = [i + BOUNDARY for i in tuple]
        # print(x1, x2, x3)
        l = [(x1, x2), (x2, x1), (x2, x3), (x3, x2), (x2, x2)]
        for value in l:
            x, y = value
            for k in range(0, BOX, 1):
                for z in range(0, BOX, 1):
                    figure.putpixel((x * BOX + k, y * BOX + z), (0, 0, 0))
            temp_list = []
            for i in range(-2, 2):
                temp_list.append((x+i, y-2))
            for i in range(-2, 3):
                temp_list.append((x-2, y+i))
                temp_list.append((x+2, y+i))
            for i in range(-2, 2):
                temp_list.append((x+i, y+2))
            for value in temp_list:
                i, j = value
                for k in range(0, BOX, 1):
                    for z in range(0, BOX, 1):
                        figure.putpixel((i * BOX + k, j * BOX + z), (0, 0, 0))
        if key != -1:
            for value in l:
                x, y = value
                for i in range(-2, 3):
                    for j in range(-2, 3):
                        for k in range(0, BOX, 1):
                            for z in range(0, BOX, 1):
                                figure.putpixel(((x+i) * BOX + k, (y+j) * BOX + z),
                                                (0, 0, 0))
    else:
        return False
    return figure


def draw_mask(figure=None):
    """

    :param figure:
    :return:
    """
    temp_list = []
    for i in range(2*BOUNDARY+SIZE):
        for j in range(BOUNDARY):
            temp_list.append((i, j))
            temp_list.append((i, j+SIZE+BOUNDARY))
    for i in range(BOUNDARY):
        for j in range(BOUNDARY, BOUNDARY+SIZE):
            temp_list.append((i, j))
            temp_list.append((i+BOUNDARY+SIZE, j))
    for value in temp_list:
        i, j = value
        for k in range(0, BOX, 1):
            for z in range(0, BOX, 1):
                figure.putpixel((i * BOX + k, j * BOX + z), (0, 0, 0))
    draw_position_detection_pattern(figure, key=1)
    draw_alignment_pattern(figure, key=1)
    return figure


def stand_point(figure=None):
    """
    标定数据点
    :return:
    """
    global data_list_left, data_list_right
    data_list_left = []
    data_list_right = []
    img_array = figure.load()
    for i in range(BOUNDARY, SIZE + BOUNDARY, 1):
        for j in range(BOUNDARY, SIZE + BOUNDARY, 1):
            if img_array[i * BOX, j * BOX] == (255, 255, 255):
                # figure.putpixel((i*BOX, j*BOX), (0, 0, 0)) # 写入单个像素
                if i < (BOUNDARY + SIZE) // 2 + 2:
                    figure.putpixel((i * BOX, j * BOX), (255, 0, 0))
                    data_list_left.append((i * BOX, j * BOX))
                    # for k in range(0, BOX, 1):
                    #     for z in range(0, BOX, 1):
                    #         figure.putpixel((i * BOX + k, j*BOX + z), (255, 255, 0))
    data_list_left = list(reversed(data_list_left))  # 生成左侧编码位需要
    for i in range(BOUNDARY, SIZE + BOUNDARY, 1):
        for j in range(BOUNDARY, SIZE + BOUNDARY, 1):
            if img_array[i * BOX, j * BOX] == (255, 255, 255):
                # figure.putpixel((i*BOX, j*BOX), (0, 0, 0)) # 写入单个像素
                if i > (BOUNDARY + SIZE) // 2 + 2:
                    figure.putpixel((i * BOX, j * BOX), (255, 0, 0))
                    data_list_right.append((i * BOX, j * BOX))
                    # for k in range(0, BOX, 1):
                    #     for z in range(0, BOX, 1):
                    #         figure.putpixel((i * BOX + k, j*BOX + z), (255, 0, 0))
    print('data_list_right: ', list(data_list_right))
    print('data_list_right 长度: ', len(list(data_list_right)))
    print('data_list_left: ', list(data_list_left))
    print('data_list_left 长度: ', len(list(data_list_left)))
    return figure


def prepare_moudle(version=-1, size=-1):
    """
    准备相应的二维码识别模板
    仅支持version 0~10
    :param version: 二维码版本
    :param size:
    :return: 二维码数据模板,左侧坐标，右侧坐标, 标线坐标
    """
    global VERSION, SIZE
    VERSION = version
    SIZE = size

    number = version

    if VERSION > 10 or VERSION < 0:
        return False
    length_of_qrcode = SIZE * BOX + BOUNDARY * BOX * 2
    img = Image.new('RGB', (length_of_qrcode, length_of_qrcode),
                    (255, 255, 255))
    print('白板生成成功！')
    img.save(dataPath + '_白板.png')  # 不允许注释
    img = draw_position_detection_pattern(img)
    img = draw_alignment_pattern(figure=img)
    moudle = deepcopy(img)
    img.save(dataPath + str(number) + '二维码识别模板.png')
    moudleFilePath = dataPath + str(number) + '二维码识别模板.png'
    print('二维码识别模板生成成功！')
    mask = draw_mask(img)
    print('二维码识别掩码生成成功！')
    mask.save(dataPath + str(number) + '二维码识别掩码.png')
    mask = stand_point(mask)
    mask.save(dataPath + str(number) + '二维码填充效果.png')
    # moudle.show()
    return moudle, moudleFilePath, data_list_left, data_list_right, standard_line


def expand_fig(multiplying_power=1.250, version=-1, filePath=''):
    """
    水平拉伸图片
    1 mm = 10 像素
    1 像素 = 0.1 mm
    :param multiplying_power:
    :return:
    """

    print('进行二维码拓展 %s...' % (str(multiplying_power), ))
    INPUT_IMAGE = Image.open(filePath)
    w1, h1 = INPUT_IMAGE.size
    img_array = INPUT_IMAGE.load()
    if w1 != h1:
        print('扩展图片失败！')
        return None
    # print(w1, h1)  # 260 260
    pixel = w1
    a = -1   # 平面二维码数据点距离中线位置(像素) multiplying_power=1.250 时，a = 115 极限
    l = 350  # 固定值（mm）
    r = 0.1 * h1 / multiplying_power  # 圆柱体半径（mm）
    x = [0.1 * i for i in range(0, pixel // 2, 1)]  # （mm）
    y = []
    for i in x:
        # 将移动的单位换算为像素
        y.append((math.asin(((r + l) / r) * math.sin(math.atan(i / l))
                            ) - math.atan(i / l)) * r * 10 - i * 10)
    y.reverse()
    # print(len(y), '\n', y)

    # 读入空白图片
    OUT_IMAGE = Image.open(dataPath + '_白板.png')
    # 扩展图片尺寸
    OUT_IMAGE = OUT_IMAGE.resize(
        (2 * round(y[0]) + pixel, pixel), Image.ANTIALIAS)
    # print(OUT_IMAGE.SIZE[0], OUT_IMAGE.SIZE[1])  # 284 260
    w2, h2 = OUT_IMAGE.size
    # 向左侧写入数据
    z = 0
    for i in range(0, w1 // 2, 1):
        for j in range(0, h1, 1):
            if img_array[i, j] == (0, 0, 0, 255):
                OUT_IMAGE.putpixel(
                    (i - round(y[z]) + round(y[0]), j), (0, 0, 0))
        z += 1
    z = 0
    # 向右侧写入数据
    for i in range(w1 - 1, w1 // 2, -1):
        for j in range(0, h1, 1):
            if img_array[i, j] == (0, 0, 0, 255):
                OUT_IMAGE.putpixel(
                    (i + round(y[z]) + round(y[0]), j), (0, 0, 0))
        z += 1
    out_img_array = OUT_IMAGE.load()
    # 填补空白
    for i in range(w2):
        for j in range(h2):
            if out_img_array[i, j] == (0, 0, 0) and out_img_array[i + 2, j] == (0, 0, 0):
                OUT_IMAGE.putpixel((i + 1, j), (0, 0, 0))
    filePath = dataPath + str(version) + '扩展图片.png'
    print('扩展二维码成功 %s %s.' % (OUT_IMAGE, filePath))
    return OUT_IMAGE, filePath


if __name__ == "__main__":
    pass



# coding: utf-8
# !/usr/bin/python
"""
@File       :   GUI_view.py
@Author     :   jiaming
@Modify Time:   2020/1/16 18:54
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   None
"""
import os
from PIL import Image
import matplotlib.pyplot as plt


rawPath = os.path.abspath(__file__)
dataPath = rawPath[:rawPath.find('view')] + 'static\\'


def window(file0Path='', file1Path='', file2Path=''):
    """

    :param time:
    :return:
    """
    titles = [file0Path, file1Path, file2Path, ""]
    print(titles)
    length = len([i for i in titles if i != ''])
    try:
        for i in range(length):
            plt.subplot(2, 2, i + 1)
            plt.imshow(Image.open(titles[i]))
            if i == 0:
                plt.title("oridinary QRcode")
            elif i == 1:
                plt.title("ac QRcode %s (not expanding)" % file1Path[
                    file1Path.rfind('\\'):][1])
            elif i == 2:
                plt.title("")
            # plt.xticks([])
            # plt.yticks([])
    except (FileExistsError, FileNotFoundError):
        print('文件不存在！')
        return None
    else:
        FILE_NAME = r'\overall_'
        FILE = dataPath + FILE_NAME + ".png"
        plt.savefig(FILE, dpi=1000, bbox_inchs='tight')
        plt.show()


if __name__ == "__main__":
    # import doctest
    # doctest.testmod()
    window()



# coding: utf-8
# !/usr/bin/python
"""
@File       :   run.py
@Author     :   jiaming
@Modify Time:   2020/2/27 19:49
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   None
"""
import os
import time
from PIL import Image
import pretty_errors
import pysnooper

from cqrcode.app.static_data import data64, data128, data256, \
    dataPath, standard_module_table, Alphanumeric_mode_table, capacity_of_each_qrcode
from cqrcode.app.prepare_data import data_decode, data_encode, \
    random_bit
from cqrcode.app.prepare_figure import prepare_moudle, expand_fig
from cqrcode.app.load_data_to_figure import load_data
from cqrcode.app.scan_figure import scan_cylinder_qr_code

# 二维码版本
VERSION = 10
# 二维码尺寸
SIZE = (VERSION - 1) * 4 + 21
# 每个像素由 4*4 个像素点构成
BOX = 4
# 边界距离 单位
BOUNDARY = 4
data_list_left = []
data_list_right = []
standard_line = []


def create_plane_qrcode(data='', moudulefig=None):
    """
    生成平面二维码
    :param data:
    :param filePath:
    :return:
    """
    print('数据编码中...')
    s = data_encode(alpha=data)
    data_string = ''
    for i in range(len(data_list_left) // len(s)):
        data_string += s
    data_string += random_bit(length=len(data_list_left)-len(data_string))
    figure, filePath = load_data(figure=moudulefig, data=data_string,
                                 version=VERSION, box=BOX,
                                 data_left=data_list_left, data_right=data_list_right)
    figure.save(filePath, dpi=(254.0, 254.0))  # 生成图片
    print('===\n柱形二维码构建完毕. %s %s \n' % (figure, filePath))
    return figure, filePath


def check(data, version=None):
    """
    检查数据以及version是否正确
    :param data:
    :param version:
    :return:
    """
    # 检查数据是否正确
    for i in data.upper():
        if i not in Alphanumeric_mode_table:
            return False
    if version is None:
        for k, v in capacity_of_each_qrcode.items():
            if v >= len(data):
                return k
    else:
        # 检查版本
        if version < 0 or version > 10:
            return False

    return True


# @pysnooper.snoop()
def main(data=data64, version=None, box=4, Boundary=4, exps=True, rate=1.250):
    """

    :param data:
    :param version:
    :param box:
    :param Boundary:
    :param exps: False 不进行扩展 True 进行扩展
    :param rate: 二维码编程 / 柱体半径
    :return:
    """
    if check(data, version) is False:
        print("数据或版本出现错误！")
        return False
    elif version is None:
        version = check(data, version)
        print('自适应版本二维码: ', version)
    global VERSION, SIZE, BOUNDARY, BOX
    VERSION = version
    SIZE = (VERSION - 1) * 4 + 21
    BOUNDARY = Boundary
    BOX = box
    # 生成二维码识别模板
    global data_list_left, data_list_right, standard_line
    moudle, moudleFilePath, data_left, data_right, std_line = prepare_moudle(
        version=VERSION, size=SIZE)
    data_list_left = data_left
    data_list_right = data_right
    standard_line = std_line
    # 生成平面二维码
    figure, plane_qecode_filePath = create_plane_qrcode(data=data,
                                                   moudulefig=moudle)
    # 识别平面二维码
    scan_cylinder_qr_code(data_left=data_list_left, data_right=data_list_right,
                          result=data, filePath=plane_qecode_filePath)
    # 进行二维码扩展
    if exps is True and 1.5 >= rate > 0:
        expand_figure, expand_filePath = expand_fig(multiplying_power=rate,
                                        version=VERSION,
                                       filePath=plane_qecode_filePath)
        expand_figure.save(expand_filePath, dpi=(254.0, 254.0))  # 生成图片
        return plane_qecode_filePath, expand_filePath
    else:
        print('扩展失败！')
        return plane_qecode_filePath


if __name__ == "__main__":
    # import doctest
    # doctest.testmod()
    main()


# coding: utf-8
# !/usr/bin/python
"""
@File       :   randomStr.py
@Author     :   jiaming
@Modify Time:   2020/2/14 13:14
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   集合了字符编码解码函数调用
                只能对于 Alphanumeric_mode_table 中的字符进行编码
                进行编码以及节码

"""
import re
import random
from app.static_data import number_of_bits_in_character_count, Alphanumeric, \
    Alphanumeric_mode_table


def data_encode(alpha=''):
    """
    对传入原生字符串进行检查并编码为一份标准填充比特流。
    :param alpha: Alphanumeric_mode_table 表中的字符
    :return: 原生字符对应的填充比特流
    """
    alpha = alpha.upper()
    for i in alpha:
        if i not in Alphanumeric_mode_table.keys():
            print('ValueError 不支持字符！')
            return False
    alpha_group = ''
    results = ''  # 保存最终比特流

    # 对于原生字符，两两成组，转换为 11 bit
    for i in range(0, len(alpha) - 1, 2):
        alpha_group += alpha[i] + alpha[i + 1] + ' '
        number = Alphanumeric_mode_table[alpha[i]] * 45 + \
            Alphanumeric_mode_table[alpha[i + 1]]
        bits = ''.join(list(bin(number))[2:])
        # 不够 11 bit， 用 0 补齐。
        if len(bits) < 11:
            bits = '0' * (11 - len(bits)) + bits  # 得到原始数据
        results += bits + ' '

    # 对于落单的字符单独编成 6 bit 数据
    if len(alpha) % 2 != 0:
        alpha_group += alpha[-1]
        number = Alphanumeric_mode_table[alpha[-1]]
        bits = ''.join(list(bin(number))[2:])
        if len(bits) < 6:
            bits = '0' * (6 - len(bits)) + bits  # 得到原始数据
        results += bits + ' '

    number_of_bits = ''.join(list(bin(len(alpha)))[2:])
    if len(number_of_bits) < number_of_bits_in_character_count:
        number_of_bits = '0' * \
            (number_of_bits_in_character_count - len(number_of_bits)) + number_of_bits

    print('消除空格前编码后数据： ', Alphanumeric + ' ' + number_of_bits + ' ' + results +
          '0000')
    data_bits = (Alphanumeric + ' ' + number_of_bits + ' ' + results +
                 '0000').replace(' ', '')
    print('消除空格后编码后数据: ', data_bits)
    return data_bits


def cut_text(text, length):
    """
    按照 lenth 的大小分隔 text 字符串
    :param text:
    :param lenth:
    :return: ['123', '456', '12']
    """
    textArr = re.findall('.{' + str(length) + '}', text)
    textArr.append(text[(len(textArr) * length):])
    return textArr


def get_key(value):
    """
    根据字典的值，返回键
    :param value:
    :return:
    """
    for k, v in Alphanumeric_mode_table.items():
        if v == value:
            return k


def data_decode(data_bits='', result=''):
    """
    数据解码，对于原生字符串编码后的数据进行解码
    :param data_bits: 编码后的比特流
    :param result: 解码的正确结果
    :return:
    """
    # 去除编码模式，因为我们默认为字符编码，占 4 bits —— 1101
    # 我们默认是版本 10，故，两个字符占用 11 bits
    data_bits = data_bits[number_of_bits_in_character_count + 4:-4]  # 取出数据位
    print('原生字符串中数据的编码结果：', data_bits)
    print('数据分段显示： ', cut_text(data_bits, 11))  # ['00111001110',
    # '11100111001', '000010']
    data_list = cut_text(data_bits, 11)
    if data_list[-1] == '':
        data_list = data_list[:-1]
    data = ''
    for i in data_list:
        # 11 bit 的数据解码
        if len(i) == number_of_bits_in_character_count:
            alpha1 = get_key(int(i, 2) // 45)
            alpha2 = get_key(int(i, 2) % 45)
            data += alpha1 + alpha2
            print('*解码数据： ', alpha1 + alpha2)
        # 6 bit 的数据解码
        elif len(i) == 6:
            alpha3 = get_key(int(i, 2) % 45)
            data += alpha3
            print('*解码数据： ', alpha3)
        # 否则出错
        else:
            raise RuntimeError('解码运行出错！', i)
    print('##\n最终识别结果： %s\n' % (data,))
    if data == result.upper():
        # print('解码结果：Successed!')
        return True
    else:
        # print('解码结果 Failed!')
        # print(data)
        return False


def random_bit(length=1459):
    """
    返回 length 长度的比特流
    :param length:
    :return:
    """

    # s = """
    # 上邪！我欲与君相知，长命无绝衰。
    # 山无陵，江水为竭，冬雷震震，夏雨雪，天地合，乃敢与君绝。
    # """
    # return encode_string(s)
    randomNum = ''.join([random.choice(['1', '0']) for i in range(length)])
    return randomNum


if __name__ == "__main__":
    data_encode()



# coding: utf-8 
# !/usr/bin/python
"""
@File       :   QRcode.py
@Author     :   jiaming
@Modify Time:   2020/1/13 19:55    
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   生成二维码尺寸：Version 10，260x260
"""
import os
import sys
import time
import pretty_errors
import pysnooper
import qrcode
from PIL import Image
from pyzbar import pyzbar
# from app.randomStr import data64, data128, data256


rawPath = os.path.abspath(__file__)
dataPath = rawPath[:rawPath.find('app')] + r'static'


# @pysnooper.snoop()
def load_data():
    """
    读入 static/input.txt 文本内容作为二维码的填充数据
    :return:
    """
    global dataPath
    file = dataPath + r'\_input.txt'
    with open(file, 'r', encoding='utf-8') as f:
        dataString = f.read()
    return dataString


# @pysnooper.snoop()
def create_QRcode():
    """
    创建二维码, 并保存到 static/ 下
    :return: 生成的二维码路径
    """
    print('生成传统二维码')
    # 导入二维码填充数据
    string = load_data()

    # 生成二维码路径以及文件名
    # TIME = time.strftime('_%Y_%m_%d_%H_%M_%S', time.localtime(time.time()))
    TIME = ''
    FILE_NAME = '\传统二维码'
    FILE = dataPath + FILE_NAME + TIME + ".png"

    # 向二维码中填充数据
    """
    version：值为1~40的整数，控制二维码的大小（最小值是1，是个12×12的矩阵）。 如果想让程序自动确定，将值设置为 None 并使用 fit 参数即可。
    error_correction：控制二维码的错误纠正功能。可取值下列4个常量。
        ERROR_CORRECT_L：大约7%或更少的错误能被纠正。
        ERROR_CORRECT_M（默认）：大约15%或更少的错误能被纠正。
        ROR_CORRECT_H：大约30%或更少的错误能被纠正。
        ERROR_CORRECT_Q:至多能够矫正25%的错误。
    box_size：控制二维码中每个小格子包含的像素数。
    border：控制边框（二维码与图片边界的距离）包含的格子数（默认为4，是相关标准规定的最小值）。
    """
    qr = qrcode.QRCode(
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=4,
        border=4,
    )  # 设置图片格式
    data = string
    qr.add_data(data)
    qr.make(fit=True)
    # 生成二维码
    img = qr.make_image(fill_color='black', back_color='white')
    # img.save(FILE, dpi=(254.0, 254.0))  # 生成图片
    # im = Image.open(FILE)
    # os.remove(FILE)
    # im.save(FILE, dpi=(254.0, 254.0))
    return img, FILE
    # os.remove(QRImagePath)  # 删除图片


# @pysnooper.snoop()
def decode_QRcode(code_img_path):
    """
    :param code_img_path:
    :return: 打印出识别的结果
    """
    decode_data = pyzbar.decode(Image.open(code_img_path), symbols=[
        pyzbar.ZBarSymbol.QRCODE])[0].data.decode('utf-8')
    return decode_data


# @pysnooper.snoop()
def main():
    pass


if __name__ == "__main__":
    # import doctest
    # doctest.testmod()
    # main()
    path = create_QRcode()
    # print(decode_QRcode(path[0]))
    # print(decode_QRcode("E:\Programmer\PYTHON\graduation_project\static"
    #                "\create_QRcode_2020_01_13_15_04_05.png"))


# coding: utf-8 
# !/usr/bin/python
"""
@File       :   __init__.py.py
@Author     :   jiaming
@Modify Time:   2020/3/8 22:46    
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   None
"""

from cqrcode.view.GUI_welcome import hello

while True:
    x = input('请输入 s 启动...\n')
    if x == 's' or x == 'S':
        hello()



# coding: utf-8 
# !/usr/bin/python
"""
@File       :   scan_figure.py
@Author     :   jiaming
@Modify Time:   2020/3/9 14:41    
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   None
"""
import cv2
import os
import time
import numpy as np
from PIL import Image

from cqrcode.app.prepare_data import data_decode
from cqrcode.app.static_data import dataPath, \
    number_of_bits_in_character_count, coordinate_no_expanding_table

data_list_left = []
data_list_right = []


def scan_left_qrcode(filePath=None, result=''):
    """
    扫描左侧数据
    :param filePath:
    :return:
    """
    print('===\n开始扫描左侧.\n')
    figure = Image.open(filePath)
    img_array = figure.load()
    # 识别左侧区域
    temp = ''
    for i in range(number_of_bits_in_character_count):
        if img_array[data_list_left[4+i][0], data_list_left[4+i][1]]  == (
                255, 255, 255, 255):
            temp += '0'
        elif img_array[data_list_left[4+i][0], data_list_left[4+i][1]]  == (
                0, 0, 0, 255):
            temp += '1'
    print('左侧识别字符个数： ', int(temp, 2))
    if int(temp, 2) % 2 == 0:
        length = int(temp, 2) // 2 * 11
    else:
        length = (int(temp, 2) - 1) // 2 * 11 + 6
    print('左侧识别字符数据 bit 数： ', length)
    temp = ''
    for i in range(length + 4 + 4 + number_of_bits_in_character_count):
        if img_array[data_list_left[i][0], data_list_left[i][1]] == (
                255, 255, 255, 255):
            temp += '0'
        elif img_array[data_list_left[i][0], data_list_left[i][1]] == (
                0, 0, 0, 255):
            temp += '1'
    print('左侧识别字符数据的 bit 流: ', temp)
    return data_decode(temp, result)


def scan_right_qrcode(filePath=None, result=''):
    """
    扫描右侧数据
    :param filePath:
    :return:
    """
    print('===\n开始扫描右侧.\n')
    figure = Image.open(filePath)
    img_array = figure.load()
    # 识别右侧区域
    temp = ''
    for i in range(number_of_bits_in_character_count):
        if img_array[data_list_right[4 + i][0], data_list_right[4 + i][1]] == (
                255, 255, 255, 255):
            temp += '0'
        elif img_array[
            data_list_right[4 + i][0], data_list_right[4 + i][1]] == (
                0, 0, 0, 255):
            temp += '1'
    print('右侧识别字符个数： ', int(temp, 2))
    if int(temp, 2) % 2 == 0:
        length = int(temp, 2) // 2 * 11
    else:
        length = (int(temp, 2) - 1) // 2 * 11 + 6
    print('右侧识别字符数据 bit 数： ', length)
    temp = ''
    for i in range(length + 4 + 4 + number_of_bits_in_character_count):
        if img_array[data_list_right[i][0], data_list_right[i][1]] == (
                255, 255, 255, 255):
            temp += '0'
        elif img_array[data_list_right[i][0], data_list_right[i][1]] == (
                0, 0, 0, 255):
            temp += '1'
    print('右侧识别字符数据的 bit 流: ', temp)
    return data_decode(temp, result)


def scan_cylinder_qr_code(data_left=[], data_right=[],
                          result='', filePath=None, real=False):
    """

    :param data_list_left:
    :param data_list_right:
    :param result:
    :param filePath:
    :return:
    """
    global data_list_left, data_list_right
    #TODO: 默认： BOX=4, BOUNDARY=4
    BOX = 4
    BOUNDARY = 4
    dict_version2size = {}
    if real is True:
        size_table = [((i-1)*4+21)*BOX+2*BOUNDARY*BOX for i in [j for j in
                                                            range(0, 11, 1)]]
        for i in range(0, 11):
            dict_version2size[i] = size_table[i]
        print(dict_version2size)
        # {0: 100, 1: 116, 2: 132, 3: 148, 4: 164, 5: 180, 6: 196, 7: 212, 8: 228, 9: 244, 10: 260}
        img = Image.open(filePath)
        w, h = img.size
        for k, v in dict_version2size.items():
            if v == w:
                version = k
                data_list_left, data_list_right = coordinate_no_expanding_table[version]
                if scan_left_qrcode(filePath, result) or scan_right_qrcode(
                        filePath, result):
                    print('##\n识别完毕\n')
                else:
                    print('##\n识别完毕\n')
                break
        else:
            print('扫描出错！')
    else:
        global data_list_left, data_list_right
        data_list_left = data_left
        data_list_right = data_right
        if scan_left_qrcode(filePath, result) or scan_right_qrcode(filePath, result):
            print('##\n识别成功\n')
        else:
            print('##\n识别失败\n')


def binary_corrode_fig(filePath=dataPath + 'target.png', threshold=200):
    """
    二值化图片
    英文路径！
    :return:
    """
    if os.path.exists(filePath) is False:
        print("文件不存在！")
        return None
    # 二值化
    print('图像二值化...')
    time.sleep(1)
    img = cv2.imread(filePath, 0)
    ret, thresh = cv2.threshold(img, threshold, 255, cv2.THRESH_BINARY)
    cv2.imwrite(dataPath + 'binary_figure.jpg', thresh)
    # 腐蚀膨胀
    print('图像腐蚀...')
    time.sleep(1)
    print('图像膨胀...')
    time.sleep(1)
    img = cv2.imread(dataPath + 'binary_figure.jpg', 0)
    kernel = np.ones((4, 4), np.uint8)
    erode = cv2.erode(img, kernel)
    # img = cv2.imread(dataPath+'binary_figure.jpg', 0)
    kernel = np.ones((3, 3), np.uint8)
    new = cv2.dilate(erode, kernel, iterations=1)
    cv2.imwrite(dataPath + 'erode_expanding_figure.jpg', new)
    os.remove(dataPath + 'binary_figure.jpg')
    # 二值化
    print('图像二值化...')
    time.sleep(1)
    img = cv2.imread(dataPath + 'erode_expanding_figure.jpg', 0)
    ret, thresh = cv2.threshold(img, threshold, 255, cv2.THRESH_BINARY)
    cv2.imwrite(dataPath + '图像二值化腐蚀膨胀.jpg', thresh)
    os.remove(dataPath + 'erode_expanding_figure.jpg')


# coding: utf-8 
# !/usr/bin/python
"""
@File       :   load_data_to_figure.py
@Author     :   jiaming
@Modify Time:   2020/3/9 15:05    
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   填充二维码
"""
import os
from PIL import Image

from app.static_data import dataPath

data_list_left = []
data_list_right = []

BOX = -1
VERSION = -1


def load_data_left(data='', figure=None):
    """
    向模板中（左侧）写入数据
    :return:
    """
    if data == '':
        print('数据为空，错误！')
        return False
    index = 0
    # figure = Image.open(filePath)
    figure = figure.convert('RGBA')
    for j in data:
        x, y = data_list_left[index]
        if j == '1':
            for k in range(0, BOX, 1):
                for z in range(0, BOX, 1):
                    figure.putpixel((x + k, y + z), (0, 0, 0))
        elif j == '0':
            for k in range(0, BOX, 1):
                for z in range(0, BOX, 1):
                    figure.putpixel((x + k, y + z), (255, 255, 255))
        index += 1
    fileName = "左侧填充结果.png"
    figure.save(dataPath + fileName, "png")
    return dataPath + fileName


def load_data_right(data='', filePath=''):
    """
    向模板中（右侧）写入数据
    :return:
    """
    if data == '':
        print('数据为空，错误！')
        return False
    index = 0
    figure = Image.open(filePath)
    figure = figure.convert('RGBA')
    for z in data:
        x, y = data_list_right[index]
        if z == '1':
            for i in range(0, BOX, 1):
                for j in range(0, BOX, 1):
                    figure.putpixel((x + i, y + j), (0, 0, 0))
        elif z == '0':
            for i in range(0, BOX, 1):
                for j in range(0, BOX, 1):
                    figure.putpixel((x + i, y + j), (255, 255, 255))
        index += 1
    fileName = str(VERSION)+"填充结果.png"
    os.remove(filePath)
    return figure, dataPath + fileName


def load_data(figure=None, data='', version=-1, box=-1, data_left=-1,
              data_right=-1):
    """

    :param data:
    :param box:
    :param data_left:
    :param data_right:
    :return:
    """
    print('data 长度: ', len(data))
    print('data_left 长度: ', len(data_left))
    print('data_right 长度: ', len(data_right))
    global BOX, data_list_left, data_list_right, VERSION
    BOX = box
    data_list_left = data_left
    data_list_right = data_right
    VERSION = version

    if data == '':
        print('数据为空，不合法！')
        return False

    print('平面二维码左侧数据的比特流: ', data)
    print('左侧写入中...')
    left_filePath = load_data_left(data, figure)
    print('平面二维码右侧数据的比特流: ', data)
    print('右侧写入中...')
    result_figure, result_filePath = load_data_right(data, left_filePath)

    # result_figure.save(result_filePath)
    return result_figure, result_filePath


# coding: utf-8 
# !/usr/bin/python
"""
@File       :   msg.py
@Author     :   jiaming
@Modify Time:   2019/12/17 15:03    
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   提示框脚本
"""
from tkinter import messagebox


def showMsg(title='', message=''):
    messagebox.showinfo(title=title, message=message)




import setuptools

setuptools.setup(name='cqrcode',
                 version='0.7',
                 description='Generate a QR code that can adapt to the cylinder',
                 long_description=open('README.md', 'r', encoding='utf-8').read(),
                 author='jiaming',
                 author_email='837357785@qq.com',
                 url = 'https://pypi.org/project/cqrcode/',
                 license='MIT',
                 packages=setuptools.find_packages(),
                 zip_safe=False,
                 include_package_data=True,
                 install_requires = [
                    'Pillow==6.1.0',
                    'pyzbar==0.1.8',
                    'qrcode==6.1',
                    'PySnooper==0.2.8',
                    'pretty-errors==1.2.7',
                    'matplotlib==3.0.3',
                 ],
                 keywords='cylinder qrcode',
                 classifiers=[
                     "Natural Language :: Chinese (Simplified)",
                     "Development Status :: 3 - Alpha",
                     "Operating System :: OS Independent",
                     "Programming Language :: Python",
                     "Programming Language :: Python :: 3.4",
                     "Programming Language :: Python :: 3.5",
                     "Programming Language :: Python :: 3.6",
                     "Programming Language :: Python :: 3.7",
                     "License :: OSI Approved :: MIT License",
                     "Topic :: Utilities"
                 ],
)


# coding: utf-8
# !/usr/bin/python
"""
@File       :   Welcome.py
@Author     :   jiaming
@Modify Time:   2020/1/16 14:06
@Contact    :   https://blog.csdn.net/weixin_39541632
@Version    :   1.0
@Desciption :   None
"""
import os
import tkinter as tk
import os
from tkinter import filedialog

from cqrcode.app.scan_figure import scan_cylinder_qr_code
from cqrcode.app.run import main
from cqrcode.app.QRcode import create_QRcode
from cqrcode.view.GUI_view import window

scaleValue = None
rawPath = os.path.abspath(__file__)
dataPath = rawPath[:rawPath.find('view')] + r'static'


def load_data():
    """
    读入 static/_input.txt 文本内容作为二维码的填充数据
    :return:
    """
    global dataPath
    file = dataPath + r'\_input.txt'
    with open(file, mode='r', encoding='UTF-8') as f:
        dataString = f.read()
    return dataString



def hello():
    """
    键入文本初始界面
    :return:
    """
    global dataPath
    root = tk.Tk()
    root.iconbitmap(dataPath + r'\icon\welcome.ico')
    root.title("Welcome")
    root.geometry('300x260+504+248')
    root.resizable(False, False)
    # root.resizable(True, True)

    # 放置图片
    def putImage():
        """
        :return:
        """
        photoLabel = tk.Label()
        path = dataPath + r'\root\blank.png'
        bm = tk.PhotoImage(file=path)
        photoLabel.x = bm
        photoLabel['image'] = bm
        photoLabel.pack()
    putImage()

    def click_btn_scan_figure(btn=None):
        """

        :param btn:
        :return:
        """
        file_path = filedialog.askopenfilename()
        print(file_path) # 打印文件的路径
        scan_cylinder_qr_code(filePath=file_path, real=True) # 调用扫描器
    scanBtn = tk.Button(root, text='select and scan figure', bd=5, height=1,
                        relief=tk.GROOVE, width=27, activeforeground="#ffffff")
    scanBtn.config(command=lambda: click_btn_scan_figure(scanBtn))
    scanBtn.pack()

    # 键入文本按钮
    def click_btn_cmd(btn=None):
        """
        :param btn:
        :return:
        """
        input_text_path = dataPath + r'\_input.txt'
        if not os.path.exists(dataPath + r'\_input.txt'):
            with open(dataPath + r'\_input.txt', 'w') as f:
                pass
        os.startfile(input_text_path)

    editBtn = tk.Button(root, text='Type the text', bd=5, height=1,
                        relief=tk.GROOVE, width=27, activeforeground="#ffffff")
    editBtn.config(command=lambda: click_btn_cmd(editBtn))
    editBtn.pack()

    def scale_command(ev=None):
        """
        :return:
        """
        global scaleValue
        scaleValue = horizontalScale.get()

    # 滑动条
    horizontalScale = tk.Scale(root, from_=0, to=1.5, tickinterval=0.5,
                               resolution=0.01, length=200,
                               orient=tk.HORIZONTAL, command=scale_command)
    horizontalScale.set(1.25)
    horizontalScale.pack()

    # label
    tk.Label(
        root,
        text='Length / Cylindrical radius',
        font='Helvetica -11').pack()

    v = tk.IntVar()

    def callCheckbutton():
        # 设置数值
        if v.get() == 1:
            print("二维码边长 / 圆柱体半径： ", float(horizontalScale.get()))
    checkButton = tk.Checkbutton(root, variable=v, text="expand figure",
                                 offvalue=0, onvalue=1,
                                 command=callCheckbutton)
    checkButton.pack()
    v.set(0)
    def create_code_cmd(btn=None):
        """

        :return:
        """
        # 生成传统二维码
        img, FILE = create_QRcode()
        img.save(FILE, dpi=(254.0, 254.0))  # 生成图片
        if v.get() == 1:
            # 生成延展的柱体二维码
            plane_qecode_filePath, expand_filePath = main(data=load_data(),
                    version=None, box=4, Boundary=4, exps=True, rate=float(
                    horizontalScale.get()))
            window(FILE, plane_qecode_filePath, expand_filePath)
        else:
            # 生成未延展后的二维码
            plane_qecode_filePath = main(data=load_data(), version=None,
                    box=4, Boundary=4, exps=False, rate=-1)
            print(plane_qecode_filePath)
            window(FILE, plane_qecode_filePath, file2Path='')
        # showMsg(title='结果', message='创建完毕')
    # 导出按钮
    createCodeBtn = tk.Button(root, width=15, height=2, text='outPut',
                              font='Helvetica -18', fg='red',
                              activebackground='#f0f0f0', relief=tk.FLAT)
    createCodeBtn.config(command=create_code_cmd)
    createCodeBtn.pack()
    root.mainloop()


if __name__ == "__main__":
    hello()
